package com.longfellowchen.btlib;

import io.flutter.plugin.common.PluginRegistry.Registrar;

import android.app.AlertDialog.Builder;
import android.bluetooth.BluetoothDevice;
import android.content.DialogInterface;
import android.content.DialogInterface.OnDismissListener;
import android.content.Intent;
import android.os.Build.VERSION;
import android.os.Bundle;
import android.os.Looper;
import android.os.Handler;
import android.util.Log;
import io.flutter.app.FlutterActivity;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.EventChannel.EventSink;
import io.flutter.plugin.common.EventChannel.StreamHandler;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import android.app.Activity;
import io.flutter.plugin.common.PluginRegistry.RequestPermissionsResultListener;

/** BtlibPlugin */
public class BtlibPlugin implements RequestPermissionsResultListener {

    /** Plugin registration. */
    public static void registerWith(Registrar registrar) {
        BtlibPlugin.me = new BtlibPlugin(registrar);
        registrar.addRequestPermissionsResultListener(BtlibPlugin.me);
    }

    private static final String eventChannel = "event.ble.longfellowchen";
    private static final String methodChannel = "method.ble.longfellowchen";
    private EventSink eventSink = null;

    public static BtlibPlugin me = null;
    private BluetoothLeDriver scanner = null;
    Map<String, BluetoothLeDriver> bluetooth_connections = new HashMap();

    public final Activity activity;

    public BtlibPlugin(Registrar registrar) {
        this.activity = registrar.activity();
        ckeckPermissions();
        Log.d("BtlibPlugin", "onCreate.");

        new EventChannel(registrar.messenger(), eventChannel).setStreamHandler(new StreamHandler() {
            public void onListen(Object arguments, EventSink events) {
                BtlibPlugin.this.eventSink = events;
            }
            public void onCancel(Object arguments) {
                BtlibPlugin.this.eventSink = null;
            }
        });
        new MethodChannel(registrar.messenger(), methodChannel).setMethodCallHandler(new MethodCallHandler() {
            public void onMethodCall(MethodCall call, Result result) {
                if (call.method.equals("bluetooth.startScan")) {
                    BtlibPlugin.this.scanner = new BluetoothLeDriver();
                    BtlibPlugin.this.scanner.startScan();
                    result.success(Integer.valueOf(0));
                } else if (call.method.equals("bluetooth.stopScan")) {
                    if (BtlibPlugin.this.scanner != null) {
                        BtlibPlugin.this.scanner.stopScan();
                        result.success(Integer.valueOf(0));
                        BtlibPlugin.this.scanner = null;
                    }
                } else if (call.method.equals("bluetooth.connect")) {
                    String deviceAddress = (String) call.argument("address");
                    Log.d("connect",deviceAddress);
                    BluetoothLeDriver bt = bluetooth_connections.get(deviceAddress);
                    if( bt == null )
                    {
                        bt = new BluetoothLeDriver();
                        bluetooth_connections.put(deviceAddress, bt );
                    }
                    bt.connect(deviceAddress);
                    result.success(Integer.valueOf(0));
                } else if (call.method.equals("bluetooth.disconnect")) {
                    String deviceAddress = (String) call.argument("address");
                    Log.d("disconnect",deviceAddress);
                    BluetoothLeDriver bt = bluetooth_connections.get(deviceAddress);
                    if (bt != null) {
                        bt.disconnect(deviceAddress);
                        result.success(Integer.valueOf(0));
                    }
                } else if (call.method.equals("bluetooth.readCharacteristic")) {
                    String deviceAddress = (String) call.argument("address");
                    Log.d("readCharacteristic",deviceAddress);
                    BluetoothLeDriver bt = bluetooth_connections.get(deviceAddress);
                    if (bt != null)
                        bt.readCharacteristic(deviceAddress,((String) call.argument("service")),
                                ((String) call.argument("characteristic")));
                    result.success(Integer.valueOf(0));
                } else if (call.method.equals("bluetooth.writeCharacteristic")) {
                    String deviceAddress = (String) call.argument("address");
                    Log.d("writeCharacteristic",deviceAddress);
                    BluetoothLeDriver bt = bluetooth_connections.get(deviceAddress);
                    if (bt != null)
                        bt.writeCharacteristic(deviceAddress,((String) call.argument("service")),
                                ((String) call.argument("characteristic")), (byte[])call.argument("data"));
                    result.success(Integer.valueOf(0));
                } else if (call.method.equals("bluetooth.setCharacteristicNotification")) {
                    String deviceAddress = (String) call.argument("address");
                    Log.d("setCharacteristicNotification",deviceAddress);
                    BluetoothLeDriver bt = bluetooth_connections.get(deviceAddress);
                    if (bt != null)
                        bt.setCharacteristicNotification(deviceAddress,((String) call.argument("service")),
                                ((String) call.argument("characteristic")), (boolean)call.argument("enabled"));
                    result.success(Integer.valueOf(0));
                } else if (call.method.equals("bluetooth.getRSSI")) {
                    String deviceAddress = (String) call.argument("address");
                    Log.d("getRSSI",deviceAddress);
                    BluetoothLeDriver bt = bluetooth_connections.get(deviceAddress);
                    if (bt != null)
                        bt.readRemoteRssi(deviceAddress);
                    result.success(Integer.valueOf(0));
                } else {
                    result.notImplemented();
                }
            }
        });
    }

    public void ThrowEvent(String eventType) {
        if (this.eventSink != null) {
            Map<String, String> map = new HashMap();
            map.put("event", eventType);
            this.eventSink.success(map);
        }
    }

    public void ThrowEvent(String eventType, String pram1, String arg1) {
        if (this.eventSink != null) {
            final Map<String, String> map = new HashMap();
            map.put("event", eventType);
            map.put(pram1, arg1);
            final EventSink threadEventSink = eventSink;
            new Handler(Looper.getMainLooper()).post(new Runnable() {
                @Override
                public void run() {
                    threadEventSink.success(map);
                }
              });
        }
    }

    public void ThrowEvent(String eventType, String pram1, String arg1, String pram2, String arg2) {
        if (this.eventSink != null) {
            final Map<String, String> map = new HashMap();
            map.put("event", eventType);
            map.put(pram1, arg1);
            map.put(pram2, arg2);
            final EventSink threadEventSink = eventSink;
            new Handler(Looper.getMainLooper()).post(new Runnable() {
                @Override
                public void run() {
                    threadEventSink.success(map);
                }
              });
        }
    }

    public void ThrowEvent(String eventType, String pram1, String arg1, String pram2, String arg2, String pram3, String arg3) {
        if (this.eventSink != null) {
            final Map<String, String> map = new HashMap();
            map.put("event", eventType);
            map.put(pram1, arg1);
            map.put(pram2, arg2);
            map.put(pram3, arg3);
            final EventSink threadEventSink = eventSink;
            new Handler(Looper.getMainLooper()).post(new Runnable() {
                @Override
                public void run() {
                    threadEventSink.success(map);
                }
              });
        }
    }

    public byte[] hex2Byte(String hexString) {
        byte[] bytes = new byte[hexString.length() / 2];
        for (int i=0 ; i<bytes.length ; i++)
        bytes[i] = (byte) Integer.parseInt(hexString.substring(2 * i, 2 * i + 2), 16);
        return bytes;
    }

    public void ThrowEventBuff(String eventType, String pram1, String arg1, String pram2, byte[] arg2) {
        if (this.eventSink != null) {
            final Map<String, Object> map = new HashMap();
            map.put("event", eventType);
            map.put(pram1, arg1);
            map.put(pram2, arg2 );
            final EventSink threadEventSink = eventSink;
            new Handler(Looper.getMainLooper()).post(new Runnable() {
                @Override
                public void run() {
                    threadEventSink.success(map);
                }
              });
        }
    }

    public void ThrowEventBuff(String eventType, String pram1, String arg1, String pram2, String arg2, String pram3, byte[] arg3) {
        if (this.eventSink != null) {
            final Map<String, Object> map = new HashMap();
            map.put("event", eventType);
            map.put(pram1, arg1);
            map.put(pram2, arg2 );
            map.put(pram3, arg3 );
            final EventSink threadEventSink = eventSink;
            new Handler(Looper.getMainLooper()).post(new Runnable() {
                @Override
                public void run() {
                    threadEventSink.success(map);
                }
              });
        }
    }

    public void ThrowEventBuff(String eventType, String pram1, String arg1, String pram2, String arg2, String pram3, String arg3, String pram4, byte[] arg4) {
        if (this.eventSink != null) {
            final Map<String, Object> map = new HashMap();
            map.put("event", eventType);
            map.put(pram1, arg1);
            map.put(pram2, arg2 );
            map.put(pram3, arg3 );
            map.put(pram4, arg4 );
            final EventSink threadEventSink = eventSink;
            new Handler(Looper.getMainLooper()).post(new Runnable() {
                @Override
                public void run() {
                    threadEventSink.success(map);
                }
              });
        }
    }

    public void ckeckPermissions() {
        if (VERSION.SDK_INT >= 23
                && this.activity.checkSelfPermission("android.permission.ACCESS_COARSE_LOCATION") != 0) {
            Builder builder = new Builder(this.activity);
            builder.setTitle("This app requires location access");
            builder.setMessage("Please grant location access so this app can detect Bluetooth");
            builder.setPositiveButton(17039370, null);
            builder.setOnDismissListener(new OnDismissListener() {
                public void onDismiss(DialogInterface dialog) {
                    BtlibPlugin.me.activity
                            .requestPermissions(new String[] { "android.permission.ACCESS_COARSE_LOCATION" }, 1);
                }
            });
            builder.show();
        }
    }

    @Override
    public boolean onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        switch (requestCode) {
        case 1:
            if (grantResults[0] == 0) {
                Intent i = this.activity.getBaseContext().getPackageManager()
                        .getLaunchIntentForPackage(this.activity.getBaseContext().getPackageName());
                i.addFlags(67108864);
                this.activity.finish();
                this.activity.startActivity(i);
                return true;
            }
            Builder builder = new Builder(this.activity);
            builder.setTitle("Limited function");
            builder.setMessage("The application is unable to detect the device in the background because location access has not been granted");
            builder.setPositiveButton(17039370, null);
            builder.setOnDismissListener(new OnDismissListener() {
                public void onDismiss(DialogInterface dialog) {
                }
            });
            builder.show();
            return true;
        default:
            return false;
        }
    }
}
