import Flutter
import UIKit
import CoreBluetooth

enum BTEvent {
    /// IOS BT EVENTS
    static let BLE_SCAN_RESULT                    = "BLE_SCAN_RESULT";
    static let BLE_STOP_SCAN                      = "BLE_STOP_SCAN";
    static let BLE_START_SCAN                     = "BLE_START_SCAN";
    static let BLE_RSSI                           = "BLE_RSSI";
    static let BLE_SERVICES_CHARACTERISTICS       = "BLE_SERVICES_CHARACTERISTICS"
    static let BLE_CONNECTED                      = "BLE_CONNECTED";
    static let BLE_DISCONNECTED                   = "BLE_DISCONNECTED";
    static let BLE_DATA_AVAILABLE                 = "BLE_DATA_AVAILABLE";
}

enum BTUUID {    
    static let CONFIG_UUID    = "00002902-0000-1000-8000-00805f9b34fb";
}

extension Data {
  /// A hexadecimal string representation of the bytes.
  func hexEncodedString() -> String {
    let hexDigits = Array("0123456789ABCDEF".utf16)
    var hexChars = [UTF16.CodeUnit]()
    hexChars.reserveCapacity(count * 2)

    for byte in self {
      let (index1, index2) = Int(byte).quotientAndRemainder(dividingBy: 16)
      hexChars.append(hexDigits[index1])
      hexChars.append(hexDigits[index2])
    }

    return String(utf16CodeUnits: hexChars, count: hexChars.count)
  }
}

public class SwiftBtlibPlugin: NSObject, FlutterPlugin, FlutterStreamHandler, CBCentralManagerDelegate, CBPeripheralDelegate  {
  
    /** Plugin registration. */
    public static func register(with registrar: FlutterPluginRegistrar) {
        _ = SwiftBtlibPlugin(registrar:registrar)
    }

    private let eventChannel = "event.ble.longfellowchen";
    private let methodChannel = "method.ble.longfellowchen";
    private var eventSink: FlutterEventSink?
    
    /// BLUETOOTH
    //var txCharacteristicUUID : CBUUID!
    var peripherals = Array<CBPeripheral>()
    var manager : CBCentralManager!
    ///var peripheral : CBPeripheral!
    var scanTimer : Timer = Timer()
 
    var mapCBPeripheral = [String:CBPeripheral]()
    var mapGattServices = [String:CBService]()
    var mapGattCharacteristic = [String:CBCharacteristic]()
    
    var isScanning = false
    var BLERSSI : NSNumber = 0

    func convertUUIDString( string : String ) -> String
    {
        let uuid = CBUUID(string:string)
        let uuidBytes = uuid.data.withUnsafeBytes { 
            [UInt8](UnsafeBufferPointer(start: $0, count: uuid.data.count)) 
            }
        if( uuid.data.count == 2 )
        {
            return String(format: "%02X%02X",uuidBytes[0],uuidBytes[1])
        } else if( uuid.data.count == 16 )
        {
            return String(format: "%02X%02X",uuidBytes[2],uuidBytes[3])
        } else
        {
            return string
        }
    }

    init(registrar: FlutterPluginRegistrar) {
        super.init()

        let rxChannel = FlutterEventChannel(name: eventChannel, binaryMessenger: registrar.messenger())
        rxChannel.setStreamHandler(self)

        let txChannel = FlutterMethodChannel(name: methodChannel, binaryMessenger: registrar.messenger())
        txChannel.setMethodCallHandler({
            (call: FlutterMethodCall, result: FlutterResult) -> Void in
            if( call.method == "bluetooth.startScan" )
            {
                self.startScan()
                result(0)
            } else if( call.method == "bluetooth.stopScan" )
            {
                self.stopScan()
                result(0)
            } else if( call.method == "bluetooth.connect" )
            {
                let args = call.arguments as! NSDictionary
                let deviceAddress = args["address"] as! String
                if( self.mapCBPeripheral[deviceAddress] != nil )
                {
                    let peripheral = self.mapCBPeripheral[deviceAddress]!
                    self.btConnect(peripheral: peripheral);
                    result(0)
                }

            } else if( call.method == "bluetooth.disconnect" )
            {
                let args = call.arguments as! NSDictionary
                let deviceAddress = args["address"] as! String
                if( self.mapCBPeripheral[deviceAddress] != nil )
                {
                    let peripheral = self.mapCBPeripheral[deviceAddress]!
                    self.btDisconnect(peripheral)
                    result(0)
                }
            } else if( call.method == "bluetooth.readCharacteristic" )
            {
                let args = call.arguments as! NSDictionary
                let characteristicUUID = self.convertUUIDString(string: args["characteristic"] as! String)
                let serviceUUID = self.convertUUIDString(string: args["service"] as! String)
                let deviceAddress = args["address"] as! String
                if( self.mapCBPeripheral[deviceAddress] != nil )
                {
                    let peripheral = self.mapCBPeripheral[deviceAddress]!
                    let characteristicUUIDall = deviceAddress + ":" + serviceUUID + ":" + characteristicUUID
                    self.log("ios:bluetooth.readCharacteristic:"+characteristicUUIDall)
                    if( self.mapGattCharacteristic[characteristicUUIDall] != nil )
                    {
                        let characteristic = self.mapGattCharacteristic[characteristicUUIDall]!
                        peripheral.readValue(for: characteristic)
                    }
                    result(0)
                }
            } else if( call.method == "bluetooth.writeCharacteristic" )
            {
                let args = call.arguments as! NSDictionary
                let characteristicUUID = self.convertUUIDString(string: args["characteristic"] as! String)
                let serviceUUID = self.convertUUIDString(string: args["service"] as! String)
                let deviceAddress = args["address"] as! String
                if( self.mapCBPeripheral[deviceAddress] != nil )
                {
                    let peripheral = self.mapCBPeripheral[deviceAddress]!
                    let characteristicUUIDall = deviceAddress + ":" + serviceUUID + ":" + characteristicUUID
                    self.log("ios:bluetooth.writeCharacteristic:"+characteristicUUIDall)
                    if( self.mapGattCharacteristic[characteristicUUIDall] != nil )
                    {
                        let characteristic = self.mapGattCharacteristic[characteristicUUIDall]!
                        let data  = args["data"] as! FlutterStandardTypedData
                        peripheral.writeValue(data.data, for:characteristic, type: CBCharacteristicWriteType.withoutResponse)//withResponse)
                    }
                    result(0)
                }
            } else if( call.method == "bluetooth.setCharacteristicNotification" )
            {
                let args = call.arguments as! NSDictionary
                let characteristicUUID = self.convertUUIDString(string: args["characteristic"] as! String)
                let serviceUUID = self.convertUUIDString(string: args["service"] as! String)
                let deviceAddress = args["address"] as! String
                if( self.mapCBPeripheral[deviceAddress] != nil )
                {
                    let peripheral = self.mapCBPeripheral[deviceAddress]!
                    let characteristicUUIDall = deviceAddress + ":" + serviceUUID + ":" + characteristicUUID
                    self.log("ios:bluetooth.writeCharacteristic:"+characteristicUUIDall)
                    if( self.mapGattCharacteristic[characteristicUUIDall] != nil )
                    {
                        let characteristic = self.mapGattCharacteristic[characteristicUUIDall]!
                        let enabled  = args["enabled"] as! Bool
                        peripheral.setNotifyValue(enabled,for:characteristic)
                    }
                    result(0)
                }
            } else if( call.method == "bluetooth.getRSSI" )
            {
                let args = call.arguments as! NSDictionary
                let deviceAddress = args["address"] as! String
                if( self.mapCBPeripheral[deviceAddress] != nil )
                {
                    let peripheral = self.mapCBPeripheral[deviceAddress]!
                    peripheral.readRSSI()
                    result(0)
                }
            } else {
                result(FlutterMethodNotImplemented)
            }
        })

        self.manager = CBCentralManager(delegate : self, queue : nil)

    }
    
    public func onListen(withArguments arguments: Any?,
                         eventSink: @escaping FlutterEventSink) -> FlutterError? {
        self.eventSink = eventSink
        return nil
    }

    public func onCancel(withArguments arguments: Any?) -> FlutterError? {
        NotificationCenter.default.removeObserver(self)
        self.eventSink = nil
        return nil
    }
    
    /// BLUETOOTH
    // MARK: CBCentralManagerDelegate   ///
    public func centralManagerDidUpdateState(_ central: CBCentralManager) {
        var msg = ""
        switch (central.state) {
        case .poweredOff:
            msg = "CoreBluetooth BLE hardware is powered off"
            
        case .poweredOn:
            msg = "CoreBluetooth BLE hardware is powered on and ready"
            
        case .resetting:
            msg = "CoreBluetooth BLE hardware is resetting"
            
        case .unauthorized:
            msg = "CoreBluetooth BLE state is unauthorized"
            
        case .unknown:
            msg = "CoreBluetooth BLE state is unknown"
            
        case .unsupported:
            msg = "CoreBluetooth BLE hardware is unsupported on this platform"
            
        @unknown default:
            msg = "CoreBluetooth unknow errot"
        }
        log(msg)
    }
    
    public func centralManager(_ central: CBCentralManager, didDiscover peripheral: CBPeripheral,
                        advertisementData: [String : Any], rssi RSSI: NSNumber) {
        log("Peripheral discovered")
        guard peripheral.name != nil else { return }
        peripherals.append(peripheral);
        let uuid : String = peripheral.identifier.uuidString
        let cut_index = uuid.index(uuid.endIndex, offsetBy: -8 )
        let mDeviceAddress: String = String(uuid[cut_index...])
        let mDeviceName: String = peripheral.name!
        mapCBPeripheral.updateValue(peripheral, forKey: mDeviceAddress)
        guard let eventSink = self.eventSink else {
            return
        }
        let ret = ["event": BTEvent.BLE_SCAN_RESULT, "name": mDeviceName, "address" : mDeviceAddress, "rssi" : RSSI.stringValue ]
        eventSink(ret)
    }

    public func peripheral(_ peripheral: CBPeripheral, didReadRSSI RSSI: NSNumber, error: Error?) 
    { 
        if (error != nil) {
            log("Failed to didReadRSSI " + error!.localizedDescription)
        } else { 
            self.BLERSSI = RSSI 
            let uuid : String = peripheral.identifier.uuidString
            let cut_index = uuid.index(uuid.endIndex, offsetBy: -8 )
            let mDeviceAddress: String = String(uuid[cut_index...])
            let ret = ["event": BTEvent.BLE_RSSI, "address":mDeviceAddress, "rssi": self.BLERSSI.stringValue ]
            guard let eventSink = self.eventSink else {
                return
            }
            eventSink(ret)
        } 
    } 

    public func btConnect( peripheral: CBPeripheral )
    {
        //self.peripheral = peripheral
        //self.peripheral.delegate = self
        peripheral.delegate = self
        self.manager.connect(peripheral, options: nil)
        self.manager.stopScan()
        self.scanTimer.invalidate()
    }
    
    public func centralManager(_ central: CBCentralManager, didConnect peripheral: CBPeripheral) {
        log("Connected to peripheral")
        ///self.peripheral.discoverServices([self.serviceUUID])
        peripheral.discoverServices(nil)
        let uuid : String = peripheral.identifier.uuidString
        let cut_index = uuid.index(uuid.endIndex, offsetBy: -8 )
        let mDeviceAddress: String = String(uuid[cut_index...])
        ///let mDeviceName: String = peripheral.name!
        let ret = ["event": BTEvent.BLE_CONNECTED, "address" : mDeviceAddress ]
        guard let eventSink = self.eventSink else {
            return
        }
        eventSink(ret)
    }
    
    /// MARK: CBPeripheralDelegate    ///
    public func peripheral(_ peripheral: CBPeripheral, didDiscoverServices error: Error?) {
        if (error != nil) {
            log("Failed to discover services " + error!.localizedDescription)
            return
        }
        let services : NSArray = peripheral.services! as NSArray
        log("Service discovered count=\(services.count) services=\(services)")
        for service in services as! [CBService] {
            mapGattServices.updateValue(service, forKey: service.uuid.uuidString.uppercased())
            self.log("ios:mapGattServices add:"+service.uuid.uuidString.uppercased())
            peripheral.discoverCharacteristics(nil, for:service)
        }
    }
    
    public func peripheral(_ peripheral: CBPeripheral, didDiscoverCharacteristicsFor service: CBService, error: Error?) {
        if error != nil {
            log("Failed to discover characteristics " + error!.localizedDescription)
            return
        }
        let characteristics : NSArray = service.characteristics! as NSArray
        log("Characteristics discovered count=\(characteristics.count) characteristics=\(characteristics)")
        let uuid : String = peripheral.identifier.uuidString
        let cut_index = uuid.index(uuid.endIndex, offsetBy: -8 )
        let mDeviceAddress: String = String(uuid[cut_index...])
        for c in service.characteristics! {
            /// save all characteristics
            mapGattCharacteristic.updateValue(c, forKey: mDeviceAddress.uppercased() + ":" + service.uuid.uuidString.uppercased() + ":" + c.uuid.uuidString.uppercased())
            self.log("ios:mapGattCharacteristic add:" + service.uuid.uuidString.uppercased() + ":" + c.uuid.uuidString.uppercased())
            let ret = ["event": BTEvent.BLE_SERVICES_CHARACTERISTICS, "address":mDeviceAddress, "service": service.uuid.uuidString.uppercased(), "characteristic": c.uuid.uuidString.uppercased() ]
            guard let eventSink = self.eventSink else {
                return
            }
            eventSink(ret)

        }
    }
    
    public func peripheral(_ peripheral: CBPeripheral, didUpdateValueFor characteristic: CBCharacteristic, error: Error?) {
        if error != nil {
            log("Failed to read value " + error!.localizedDescription)
            return
        }
        /// PROCESS STRING
        log("peripheral read=\(String(describing: characteristic.value))")
        /// GET DATA
        let uuid : String = peripheral.identifier.uuidString
        let cut_index = uuid.index(uuid.endIndex, offsetBy: -8 )
        let mDeviceAddress: String = String(uuid[cut_index...])
        let data = FlutterStandardTypedData(bytes: characteristic.value! )
        let ret : NSDictionary = ["event": BTEvent.BLE_DATA_AVAILABLE, "address":mDeviceAddress, "data": data, "service": characteristic.service.uuid.uuidString.uppercased(), "characteristic": characteristic.uuid.uuidString.uppercased() ]
        guard let eventSink = self.eventSink else {
            return
        }
        eventSink(ret)
    }
    
    public func centralManager(_ central: CBCentralManager, didDisconnectPeripheral peripheral: CBPeripheral, error: Error?) {
        log("Discnnected to peripheral")
        let uuid : String = peripheral.identifier.uuidString
        let cut_index = uuid.index(uuid.endIndex, offsetBy: -8 )
        let mDeviceAddress: String = String(uuid[cut_index...])
        let ret = ["event": BTEvent.BLE_DISCONNECTED, "address" : mDeviceAddress ]
        guard let eventSink = self.eventSink else {
            return
        }
        eventSink(ret)
    }
    
    public func peripheral(_ peripheral: CBPeripheral, didWriteValueFor descriptor: CBDescriptor,
               error: Error?)
    {
        log("didWriteValueFor\(descriptor)")
    }

    public func log(_ str:String)
    {
        NSLog(str)
    }
    
    public func stopScan()
    {
        log("Stop Scan")
        if( isScanning ){
            self.manager.stopScan()
            let ret = ["event": BTEvent.BLE_STOP_SCAN ]
            guard let eventSink = self.eventSink else {
                return
            }
            eventSink(ret)
            isScanning = false
        }
    }

    /// scan timeout
    @objc func scanTimeout(){
        stopScan()
    }
    
    /// start scan
    public func startScan()
    {
        self.peripherals.removeAll()
        //self.manager.scanForPeripherals(withServices: [self.serviceUUID], options: nil)
        self.manager.scanForPeripherals(withServices: nil, options: nil)
        self.scanTimer = Timer.scheduledTimer(timeInterval: 3, target:self, selector: #selector(SwiftBtlibPlugin.scanTimeout), userInfo: nil, repeats: false)
        let ret = ["event": BTEvent.BLE_START_SCAN ]
        guard let eventSink = self.eventSink else {
            return
        }
        eventSink(ret)
        isScanning = true
    }

    /// disconnect
    public func btDisconnect(_ peripheral: CBPeripheral)
    {
        self.manager.cancelPeripheralConnection(peripheral)
        self.scanTimer.invalidate()
    }
}
