import 'dart:async';
import 'dart:typed_data';
import 'package:flutter/services.dart';

class BTEvent {
  /// FLUTTER BT EVENTS
  static const BLE_SCAN_RESULT = "BLE_SCAN_RESULT";
  static const BLE_STOP_SCAN = "BLE_STOP_SCAN";
  static const BLE_START_SCAN = "BLE_START_SCAN";
  static const BLE_RSSI = "BLE_RSSI";
  static const BLE_SERVICES_CHARACTERISTICS = "BLE_SERVICES_CHARACTERISTICS";
  static const BLE_CONNECTED = "BLE_CONNECTED";
  static const BLE_DISCONNECTED = "BLE_DISCONNECTED";
  static const BLE_DATA_AVAILABLE = "BLE_DATA_AVAILABLE";
}

class Btlib {
  static final Btlib _singleton = Btlib._internal();

  factory Btlib() {
    return _singleton;
  }

  MethodChannel _methodChannel =
      const MethodChannel("method.ble.longfellowchen");
  EventChannel _eventChannel = const EventChannel('event.ble.longfellowchen');
  StreamController<Object> _eventCtrl;

  Btlib._internal() {
    print("Btlib init");
    _eventChannel.receiveBroadcastStream().listen(_onEvent, onError: _onError);
  }

  void _onEvent(Object event) {
    _eventCtrl.sink.add(event);
  }

  void _onError(Object error) {
    print(error);
  }

  StreamSubscription<Object> listen(to) {
    if (_eventCtrl == null) _eventCtrl = StreamController<Object>.broadcast();
    return _eventCtrl.stream.listen(to);
  }

  Future startScan() async {
    try {
      print("invokeMethod bluetooth.startScan");
      await _methodChannel.invokeMethod("bluetooth.startScan");
    } on PlatformException catch (e) {
      print(e.toString());
    }
  }

  Future stopScan() async {
    try {
      print("invokeMethod bluetooth.stopScan");
      await _methodChannel.invokeMethod("bluetooth.stopScan");
    } on PlatformException catch (e) {
      print(e.toString());
    }
  }

  Future disconnect(String address) async {
    try {
      print("invokeMethod bluetooth.disconnect");
      await _methodChannel
          .invokeMethod("bluetooth.disconnect", {"address": address});
    } on PlatformException catch (e) {
      print(e.toString());
    }
  }

  Future connect(String address) async {
    try {
      print("invokeMethod bluetooth.connect");
      await _methodChannel
          .invokeMethod("bluetooth.connect", {"address": address});
    } on PlatformException catch (e) {
      print(e.toString());
    }
  }

  Future getRSSI(String address) async {
    try {
      print("invokeMethod bluetooth.rssi");
      await _methodChannel
          .invokeMethod("bluetooth.getRSSI", {"address": address});
    } on PlatformException catch (e) {
      print(e.toString());
    }
  }

  Future setCharacteristicNotification(String address, String service,
      String characteristic, bool enabled) async {
    try {
      print("invokeMethod bluetooth.setCharacteristicNotification");
      await _methodChannel
          .invokeMethod("bluetooth.setCharacteristicNotification", {
        "address": address,
        "service": service,
        "characteristic": characteristic,
        "enabled": enabled
      });
    } on PlatformException catch (e) {
      print(e.toString());
    }
  }

  Future readCharacteristic(
      String address, String service, String characteristic) async {
    try {
      print("invokeMethod bluetooth.readCharacteristic");
      await _methodChannel.invokeMethod("bluetooth.readCharacteristic", {
        "address": address,
        "service": service,
        "characteristic": characteristic
      });
    } on PlatformException catch (e) {
      print(e.toString());
    }
  }

  Future writeCharacteristic(String address, String service,
      String characteristic, Uint8List data) async {
    try {
      print("invokeMethod bluetooth.writeCharacteristic");
      await _methodChannel.invokeMethod("bluetooth.writeCharacteristic", {
        "address": address,
        "service": service,
        "characteristic": characteristic,
        "data": data
      });
    } on PlatformException catch (e) {
      print(e.toString());
    }
  }

  void dispose(filename) {
    _eventCtrl.close();
    _eventCtrl = null;
  }
}
